#include "stdafx.h"
#include "Capture.h"
#include "Image.h"


#define CAPTURE_PREVIEW_WINDOW_CLASS TEXT("HDUSTest Capture Preview")
#define TITLE_TEXT TEXT("Lv`vr[")




HINSTANCE CCapturePreview::m_hinst=NULL;


bool CCapturePreview::Initialize(HINSTANCE hinst)
{
	WNDCLASS wc;

	m_hinst=hinst;
	wc.style=CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc=WndProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hInstance=hinst;
	wc.hIcon=NULL;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground=NULL;
	wc.lpszMenuName=NULL;
	wc.lpszClassName=CAPTURE_PREVIEW_WINDOW_CLASS;
	return RegisterClass(&wc)!=0;
}


CCapturePreview::CCapturePreview()
{
	m_hwnd=NULL;
	m_hData=NULL;
	m_crBackColor=RGB(0,0,0);
	m_WindowPosition.Left=CW_USEDEFAULT;
	m_WindowPosition.Top=CW_USEDEFAULT;
	m_WindowPosition.Width=320;
	m_WindowPosition.Height=240;
}


CCapturePreview::~CCapturePreview()
{
	Destroy();
	if (m_hData!=NULL)
		GlobalFree(m_hData);
}


bool CCapturePreview::Create(HWND hwndParent,DWORD Style,DWORD ExStyle)
{
	return CreateWindowEx(ExStyle,CAPTURE_PREVIEW_WINDOW_CLASS,TITLE_TEXT,Style,
		m_WindowPosition.Left,m_WindowPosition.Top,
		m_WindowPosition.Width,m_WindowPosition.Height,
		hwndParent,NULL,m_hinst,this)!=NULL;
}


bool CCapturePreview::Destroy()
{
	if (m_hwnd!=NULL)
		DestroyWindow(m_hwnd);
	return true;
}


bool CCapturePreview::SetImage(const BITMAPINFO *pbmi,const void *pBits)
{
	SIZE_T InfoSize,BitsSize;
	BYTE *pData;

	ClearImage();
	InfoSize=CalcDIBInfoSize(&pbmi->bmiHeader);
	BitsSize=CalcDIBBitsSize(&pbmi->bmiHeader);
	m_hData=GlobalAlloc(GMEM_MOVEABLE,InfoSize+BitsSize);
	if (m_hData==NULL)
		return false;
	pData=(BYTE*)GlobalLock(m_hData);
	CopyMemory(pData,pbmi,InfoSize);
	CopyMemory(pData+InfoSize,pBits,BitsSize);
	GlobalUnlock(m_hData);
	if (m_hwnd!=NULL) {
		InvalidateRect(m_hwnd,NULL,TRUE);
		UpdateWindow(m_hwnd);
		SetTitle();
	}
	return true;
}


bool CCapturePreview::SetImage(HGLOBAL hData)
{
	ClearImage();
	m_hData=hData;
	if (m_hwnd!=NULL) {
		InvalidateRect(m_hwnd,NULL,TRUE);
		UpdateWindow(m_hwnd);
		SetTitle();
	}
	return true;
}


bool CCapturePreview::ClearImage()
{
	if (m_hData!=NULL) {
		GlobalFree(m_hData);
		m_hData=NULL;
		if (m_hwnd!=NULL) {
			InvalidateRect(m_hwnd,NULL,TRUE);
			SetTitle();
		}
	}
	return true;
}


bool CCapturePreview::HasImage() const
{
	return m_hData!=NULL;
}


bool CCapturePreview::SetPosition(int Left,int Top,int Width,int Height)
{
	if (m_hwnd!=NULL) {
		MoveWindow(m_hwnd,Left,Top,Width,Height,TRUE);
	} else {
		m_WindowPosition.Left=Left;
		m_WindowPosition.Top=Top;
		m_WindowPosition.Width=Width;
		m_WindowPosition.Height=Height;
	}
	return true;
}


bool CCapturePreview::GetPosition(int *pLeft,int *pTop,int *pWidth,int *pHeight) const
{
	if (m_hwnd!=NULL) {
		WINDOWPLACEMENT wc;

		wc.length=sizeof(WINDOWPLACEMENT);
		GetWindowPlacement(m_hwnd,&wc);
		if (pLeft)
			*pLeft=wc.rcNormalPosition.left;
		if (pTop)
			*pTop=wc.rcNormalPosition.top;
		if (pWidth)
			*pWidth=wc.rcNormalPosition.right-wc.rcNormalPosition.left;
		if (pHeight)
			*pHeight=wc.rcNormalPosition.bottom-wc.rcNormalPosition.top;
	} else {
		if (pLeft)
			*pLeft=m_WindowPosition.Left;
		if (pTop)
			*pTop=m_WindowPosition.Top;
		if (pWidth)
			*pWidth=m_WindowPosition.Width;
		if (pHeight)
			*pHeight=m_WindowPosition.Height;
	}
	return true;
}


void CCapturePreview::SetTitle()
{
	if (m_hwnd!=NULL) {
		if (m_hData!=NULL) {
			TCHAR szTitle[64];
			BITMAPINFOHEADER *pbmih;

			pbmih=static_cast<BITMAPINFOHEADER*>(GlobalLock(m_hData));
			wsprintf(szTitle,TITLE_TEXT TEXT(" - %d x %d (%d bpp)"),
				pbmih->biWidth,abs(pbmih->biHeight),pbmih->biBitCount);
			SetWindowText(m_hwnd,szTitle);
			GlobalUnlock(m_hData);
		} else
			SetWindowText(m_hwnd,TITLE_TEXT);
	}
}


CCapturePreview *CCapturePreview::GetThis(HWND hwnd)
{
	return reinterpret_cast<CCapturePreview*>(GetWindowLongPtr(hwnd,GWLP_USERDATA));
}


LRESULT CALLBACK CCapturePreview::WndProc(HWND hwnd,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_CREATE:
		{
			LPCREATESTRUCT pcs=reinterpret_cast<LPCREATESTRUCT>(lParam);
			CCapturePreview *pThis=
							static_cast<CCapturePreview*>(pcs->lpCreateParams);

			pThis->m_hwnd=hwnd;
			SetWindowLongPtr(hwnd,GWLP_USERDATA,reinterpret_cast<LONG_PTR>(pThis));
			if (pThis->m_hData!=NULL)
				pThis->SetTitle();
		}
		return 0;

	case WM_PAINT:
		{
			CCapturePreview *pThis=GetThis(hwnd);
			PAINTSTRUCT ps;
			HBRUSH hbr;
			RECT rc;

			BeginPaint(hwnd,&ps);
			hbr=CreateSolidBrush(pThis->m_crBackColor);
			GetClientRect(hwnd,&rc);
			if (pThis->m_hData!=NULL) {
				BYTE *pData;
				BITMAPINFO *pbmi;
				int DstX,DstY,DstWidth,DstHeight;
				RECT rcDest;

				pData=(BYTE*)GlobalLock(pThis->m_hData);
				pbmi=(BITMAPINFO*)pData;
				DstWidth=pbmi->bmiHeader.biWidth*rc.bottom/
												abs(pbmi->bmiHeader.biHeight);
				if (DstWidth>rc.right)
					DstWidth=rc.right;
				DstHeight=pbmi->bmiHeader.biHeight*rc.right/
												pbmi->bmiHeader.biWidth;
				if (DstHeight>rc.bottom)
					DstHeight=rc.bottom;
				DstX=(rc.right-DstWidth)/2;
				DstY=(rc.bottom-DstHeight)/2;
				if (DstWidth>0 && DstHeight>0) {
					int OldStretchBltMode;

					OldStretchBltMode=SetStretchBltMode(ps.hdc,STRETCH_HALFTONE);
					StretchDIBits(ps.hdc,DstX,DstY,DstWidth,DstHeight,
						0,0,pbmi->bmiHeader.biWidth,pbmi->bmiHeader.biHeight,
						pData+CalcDIBInfoSize(&pbmi->bmiHeader),pbmi,
						DIB_RGB_COLORS,SRCCOPY);
					SetStretchBltMode(ps.hdc,OldStretchBltMode);
				}
				GlobalUnlock(pThis->m_hData);
				rcDest.left=DstX;
				rcDest.top=DstY;
				rcDest.right=DstX+DstWidth;
				rcDest.bottom=DstY+DstHeight;
				if (ps.rcPaint.top<rcDest.top) {
					rc.left=ps.rcPaint.left;
					rc.top=ps.rcPaint.top;
					rc.right=ps.rcPaint.right;
					rc.bottom=min(rcDest.top,ps.rcPaint.bottom);
					FillRect(ps.hdc,&rc,hbr);
				}
				if (ps.rcPaint.top<rcDest.bottom && ps.rcPaint.bottom>rcDest.top) {
					rc.top=max(rcDest.top,ps.rcPaint.top);
					rc.bottom=min(rcDest.bottom,ps.rcPaint.bottom);
					if (ps.rcPaint.left<rcDest.left) {
						rc.left=ps.rcPaint.left;
						rc.right=min(rcDest.left,ps.rcPaint.right);
						FillRect(ps.hdc,&rc,hbr);
					}
					if (ps.rcPaint.right>rcDest.right) {
						rc.left=max(rcDest.right,ps.rcPaint.left);
						rc.right=ps.rcPaint.right;
						FillRect(ps.hdc,&rc,hbr);
					}
				}
				if (ps.rcPaint.bottom>rcDest.bottom) {
					rc.left=ps.rcPaint.left;
					rc.top=max(rcDest.bottom,ps.rcPaint.top);
					rc.right=ps.rcPaint.right;
					rc.bottom=ps.rcPaint.bottom;
					FillRect(ps.hdc,&rc,hbr);
				}
			} else
				FillRect(ps.hdc,&rc,hbr);
			EndPaint(hwnd,&ps);
		}
		return 0;

	case WM_NCHITTEST:
		if (DefWindowProc(hwnd,uMsg,wParam,lParam)==HTCLIENT)
			return HTCAPTION;
		break;

	case WM_DESTROY:
		{
			CCapturePreview *pThis=GetThis(hwnd);

			pThis->GetPosition(&pThis->m_WindowPosition.Left,
							   &pThis->m_WindowPosition.Top,
							   &pThis->m_WindowPosition.Width,
							   &pThis->m_WindowPosition.Height);
			pThis->m_hwnd=NULL;
		}
		return 0;
	}
	return DefWindowProc(hwnd,uMsg,wParam,lParam);
}
